/* eslint-disable @typescript-eslint/no-explicit-any */
import cls from 'classnames'
import * as React from 'react'
import { Spinner } from '@uspk-ui/react-spinner/src'
import { CheckIcon } from '@uspk-ui/react-icons/src/svgIcons'
import { ButtonIcon } from './button-icon'
import { ButtonSpinner } from './button-spinner'
import styles from './buttons.module.css'
import { IButtonProps } from './types'
import uspkStyles from '@uspk-ui/scss/lib/Text.module.css'

export const Button = React.forwardRef<
  HTMLButtonElement | HTMLAnchorElement,
  IButtonProps
>(function Button(
  {
    children,
    className,
    primary,
    secondary,
    shadow3d,
    disabled,
    backgroundColor,
    textColor,
    isFullWidth,
    size = 'lg',
    btnState = 'initial',
    icon,
    inverse,
    as,
    iconComponent,
    leftIcon,
    rightIcon,
    iconSpace,
    ...rest
  },
  ref,
) {
  const [width, setWidth] = React.useState(0)
  const [height, setHeight] = React.useState(0)
  const btnRef = React.useRef<any>(ref)

  React.useEffect(() => {
    if (btnRef?.current && btnRef?.current?.getBoundingClientRect().width) {
      setWidth(btnRef?.current?.getBoundingClientRect().width)
    }
    if (btnRef?.current && btnRef?.current?.getBoundingClientRect().height) {
      setHeight(btnRef?.current?.getBoundingClientRect().height)
    }
  }, [children])

  const loadingSpinnerProps: any = {}

  if (btnState === 'processing') {
    if (
      (secondary && inverse) ||
      (icon && inverse) ||
      (!secondary && !icon && !inverse)
    ) {
      loadingSpinnerProps.inverse = true
    }
  }

  const renderChildren = () => {
    let overlayClasses
    let srText
    if (btnState !== 'initial') {
      overlayClasses = cls(
        [styles['button-state__overlay']],
        [styles[`button-state__overlay--${btnState}`]],
        btnState === 'success' && [
          styles['button-state__overlay--success-with-svg-icon'],
        ],
      )
    }

    const buttonCheckmarkIcon: React.ReactElement<any> = (
      <span>
        <CheckIcon className={styles['button-state__success-icon']} />
      </span>
    )

    const iconWrapper = iconComponent

    return (
      <>
        {iconWrapper && { icon }}

        {btnState === 'processing' ? (
          <ButtonSpinner label="loading" placement="end">
            <Spinner color="currentColor" label="hello" />
          </ButtonSpinner>
        ) : (
          <>
            {leftIcon && (
              <ButtonIcon marginEnd={iconSpace}>{leftIcon}</ButtonIcon>
            )}
            {children}
            {rightIcon && (
              <ButtonIcon marginStart={iconSpace}>{rightIcon}</ButtonIcon>
            )}
          </>
        )}
        {btnState !== 'initial' && (
          <span className={overlayClasses}>
            {btnState === 'success' && buttonCheckmarkIcon}
          </span>
        )}
        <span>{srText}</span>
      </>
    )
  }

  const Component = as || 'button'
  const _className = cls(
    [styles['button']],
    uspkStyles['uspk-text--heading_font'],
    styles[`uspk_button__size--${size}`],
    {
      [styles['button-default']]:
        !primary && !secondary && !inverse && !disabled && !shadow3d,
      [styles['button-primary']]: primary,
      [styles['button-inverse']]: inverse,
      [styles['button-disabled']]: disabled,
      [styles['button-secondary']]: secondary,
      [styles['button-shadow3d']]: shadow3d,
      [styles['button--icon']]: icon,
    },
    { [styles[`button-state__${btnState}`]]: btnState !== 'initial' },
    className,
  )

  const buttonStyle = {
    background: backgroundColor,
    color: textColor,
    width: isFullWidth ? '100%' : btnState === 'processing' && `${width}px`,
    height: btnState === 'processing' && `${height}px`,
  }

  const isDisabled = disabled || (btnState && btnState) !== 'initial'

  const button3DShadow = (
    <>
      <span className={styles['button-shadow3d--shadow']} />
      <span className={styles['button-shadow3d--edge']} />
      <span className={styles['button-shadow3d--front']}>
        {renderChildren()}
      </span>
    </>
  )

  return (
    <>
      <Component
        className={_className}
        aria-live="assertive"
        style={buttonStyle}
        disabled={isDisabled}
        ref={btnRef}
        {...rest}
      >
        {shadow3d ? button3DShadow : renderChildren()}
      </Component>
    </>
  )
})
